@@ -610,6 +610,18 @@ NIDM_MRI_DIFFUSION_TENSOR,
 NIDM_MRI_FLOW,
 NIDM_MRI_BOLD_EVENTS]
 
+# Common isAbout URIs
+NIDM_IS_ABOUT_AGE = str(INTERLEX['ilx_0100400'])
+NIDM_IS_ABOUT_HANDEDNESS = str(OBO['PATO_0002201'])
+NIDM_IS_ABOUT_GENDER = str(INTERLEX['ilx_0101292'])
+
+# REST API constants
+NIDM_REST_NUM_SUBJECTS = 'number_of_subjects'
+NIDM_REST_MAX_AGE = 'age_max'
+NIDM_REST_MIN_AGE = 'age_min'
+NIDM_REST_GENDER = 'gender'
+NIDM_REST_AGE = 'age'
+
 
 # cannonical CDE file locations
 CDE_FILE_LOCATIONS = [
