@@ -28,19 +28,18 @@
 #
 #**************************************************************************************
 #**************************************************************************************
-import os,sys
+import sys
 import rdflib
 from rdflib import Graph, URIRef, util
 import pandas as pd
 import logging
 from nidm.core import Constants
+import nidm.experiment.CDE
 import re
 import tempfile
 from os import path
 import functools
 import hashlib
-from urllib.request import urlretrieve
-
 import pickle
 
 from joblib import Memory
@@ -470,11 +469,11 @@ def GetParticipantInstrumentDataCached(nidm_file_list: tuple ,project_id, partic
 
     return result
 
-def GetParticipantUUIDsForProject(nidm_file_list: tuple, project_id, filter, output_file=None):
-    return GetParticipantUUIDsForProjectCached(tuple(nidm_file_list), project_id, filter, output_file=None)
+def GetParticipantUUIDsForProject(nidm_file_list: tuple, project_id, filter=None, output_file=None):
+    return GetParticipantUUIDsForProjectCached(tuple(nidm_file_list), project_id, filter, output_file)
 
 @functools.lru_cache(maxsize=QUERY_CACHE_SIZE)
-def GetParticipantUUIDsForProjectCached(nidm_file_list:tuple, project_id, filter, output_file=None):
+def GetParticipantUUIDsForProjectCached(nidm_file_list:tuple, project_id, filter=None, output_file=None):
     '''
     This query will return a list of all prov:agent entity UUIDs within a single project
     that prov:hadRole sio:Subject or Constants.NIDM_PARTICIPANT
@@ -492,8 +491,8 @@ def GetParticipantUUIDsForProjectCached(nidm_file_list:tuple, project_id, filter
     ### added by DBK changed to dictionary to support subject ids along with uuids
     #participants = []
     participants = {}
-    participants["uuid"] = []
-    participants["subject id"] = []
+    participants["uuid"] = set()
+    participants["subject id"] = set()
 
 
     for file in nidm_file_list:
@@ -518,14 +517,16 @@ def GetParticipantUUIDsForProjectCached(nidm_file_list:tuple, project_id, filter
                                     ### added by DBK for subject IDs as well ###
                                     #participants.append(uuid)
                                     try:
-                                        participants['uuid'].append(uuid)
-                                        participants['subject id'].append(subid)
+                                        participants['uuid'].add(uuid)
+                                        participants['subject id'].add(subid)
                                     # just in case there's no subject id in the file...
                                     except:
                                         #participants.append(uuid)
-                                        participants['uuid'].append(uuid)
-                                        participants['subject id'].append('')
+                                        participants['uuid'].add(uuid)
+                                        participants['subject id'].add('')
 
+    participants['uuid'] = list(participants['uuid'])
+    participants['subject id'] = list(participants['subject id'])
     return participants
 
 
@@ -573,10 +574,16 @@ def GetDatatypeSynonyms(nidm_file_list, project_id, datatype):
     :param datatype:
     :return:
     '''
+    if datatype.startswith("instruments."):
+        datatype = datatype[12:]
+    if datatype.startswith("derivatives."):
+        datatype = datatype[12:]
     project_data_elements = GetProjectDataElements(nidm_file_list, project_id)
     for dti in project_data_elements['data_type_info']:
-        if str(datatype) in [ str(x) for x in [dti['label'], dti['datumType'], dti['measureOf'], URITail(dti['measureOf']), dti['isAbout'], URITail(dti['isAbout']), dti['dataElement'], dti['dataElementURI'], dti['prefix']] ]:
-            return [str(dti['label']), str(dti['datumType']), str(dti['measureOf']), URITail(dti['measureOf']), str(dti['isAbout']), str(dti['dataElement']), str(dti['dataElementURI']), str(dti['prefix'])]
+        if str(datatype) in [ str(x) for x in [dti['source_variable'], dti['label'], dti['datumType'], dti['measureOf'], URITail(dti['measureOf']), dti['isAbout'], URITail(dti['isAbout']), dti['dataElement'], dti['dataElementURI'], dti['prefix']] ]:
+            all_synonyms = set([str(dti['source_variable']), str(dti['label']), str(dti['datumType']), str(dti['measureOf']), URITail(dti['measureOf']), str(dti['isAbout']), str(dti['dataElement']), str(dti['dataElementURI'])] )
+            all_synonyms.remove("")  # remove the empty string in case that is in there
+            return list(all_synonyms)
     return [datatype]
 
 def GetProjectDataElements(nidm_file_list, project_id):
@@ -775,16 +782,16 @@ def GetProjectsMetadata(nidm_file_list):
     return {'projects': compressForJSONResponse(projects)}
 
 
-def GetProjectsComputedMetadata(nidm_file_list):
-    '''
-     :param nidm_file_list: List of one or more NIDM files to query across for list of Projects
-    :return: dataframe with two columns: "project_uuid" and "project_dentifier"
-    '''
-
-    meta_data = GetProjectsMetadata(nidm_file_list)
-    ExtractProjectSummary(meta_data, nidm_file_list)
-
-    return compressForJSONResponse(meta_data)
+# def GetProjectsComputedMetadata(nidm_file_list):
+#     '''
+#      :param nidm_file_list: List of one or more NIDM files to query across for list of Projects
+#     :return: dataframe with two columns: "project_uuid" and "project_dentifier"
+#     '''
+#
+#     meta_data = GetProjectsMetadata(nidm_file_list)
+#     ExtractProjectSummary(meta_data, nidm_file_list)
+#
+#     return compressForJSONResponse(meta_data)
 
 def GetDataElements(nidm_file_list):
 
@@ -893,74 +900,6 @@ def GetBrainVolumes(nidm_file_list):
 
 
 
-def ExtractProjectSummary(meta_data, nidm_file_list):
-    '''
-
-    :param meta_data: a dictionary of projects containing their meta data as pulled from the nidm_file_list
-    :param nidm_file_list: List of NIDM files
-    :return:
-    '''
-    query = '''
-    prefix ncicb: <http://ncicb.nci.nih.gov/xml/owl/EVS/Thesaurus.owl#>
-    prefix ndar: <https://ndar.nih.gov/api/datadictionary/v2/dataelement/>
-    prefix obo: <http://purl.obolibrary.org/obo/>
-    prefix dct: <http://purl.org/dc/terms/>
-    prefix prov: <http://www.w3.org/ns/prov#>
-    prefix nidm: <http://purl.org/nidash/nidm#>
-
-    SELECT DISTINCT ?id ?person ?age ?gender ?hand ?assessment ?acq ?session ?project
-    WHERE {
-      # Added by DBK to support new data element format
-      {?age_measure a nidm:DataElement ;
-					nidm:isAbout ncicb:Age .}
-	  {?gender_measure a nidm:DataElement ;
-					nidm:isAbout ndar:gender .}
-	  {?handedness_measure a nidm:DataElement ;
-					nidm:isAbout obo:handedness .}
-      OPTIONAL { ?assessment ?age_measure ?age } .
-      OPTIONAL { ?assessment ?gender_measure ?gender } .
-      OPTIONAL { ?assessment ?handedness_measure ?hand } .
-      ?person ndar:src_subject_id ?id .
-      ?acq prov:qualifiedAssociation _:blank .
-      _:blank prov:hadRole sio:Subject .
-      _:blank prov:agent ?person .
-      ?assessment prov:wasGeneratedBy ?acq .
-      ?acq dct:isPartOf ?session .
-      ?session dct:isPartOf ?project .
-      ?project a nidm:Project
-    }
-    ORDER BY ?id
-      '''
-
-    df = sparql_query_nidm(nidm_file_list, query, output_file=None)
-    projects = meta_data['projects']
-
-    arr = df.values
-    key = str(Constants.NIDM_NUMBER_OF_SUBJECTS)
-    for project_id, project in projects.items():
-        project[key] = 0
-        project['age_max'] = 0
-        project['age_min'] = sys.maxsize
-        project[str(Constants.NIDM_GENDER)] = []
-        project[str(Constants.NIDM_HANDEDNESS)] = []
-
-    for row in arr:
-        project_id = matchPrefix( str(row[8]) ) # 9th column is the project UUID
-        projects[project_id][str(Constants.NIDM_NUMBER_OF_SUBJECTS)] += 1
-
-        age = float(row[2])
-        projects[project_id]['age_min'] = min(age, projects[project_id]['age_min'])
-        projects[project_id]['age_max'] = max(age, projects[project_id]['age_max'])
-
-        gender = str(row[3])
-        if gender not in projects[project_id][str(Constants.NIDM_GENDER)]:
-            projects[project_id][str(Constants.NIDM_GENDER)].append(gender)
-
-        hand = str(row[4])
-        if hand not in projects[project_id][str(Constants.NIDM_HANDEDNESS)]:
-            projects[project_id][str(Constants.NIDM_HANDEDNESS)].append(hand)
-
-
 def expandNIDMAbbreviation(shortKey) -> str:
     '''
     Takes a shorthand identifier such as dct:description and returns the 
@@ -1083,7 +1022,7 @@ def getDataTypeInfo(source_graph, datatype):
     if source_graph and  (expanded_datatype, isa, Constants.NIDM['DataElement']) in source_graph:
         rdf_graph = source_graph
     else:
-        rdf_graph = getCDEs()
+        rdf_graph = nidm.experiment.CDE.getCDEs()
 
     typeURI = ''
     hasUnit = ''
@@ -1093,6 +1032,7 @@ def getDataTypeInfo(source_graph, datatype):
     isAbout = ''
     structure = ''
     prefix = ''
+    source_variable = ''
 
     found = False
 
@@ -1102,6 +1042,8 @@ def getDataTypeInfo(source_graph, datatype):
         found = True
         if (re.search(r'label$', str(p)) != None):
             label = o
+        if (re.search(r'source_variable$', str(p)) != None):
+            source_variable = o
         if (re.search(r'description$', str(p)) != None):
             description = o
         if (re.search(r'hasUnit$', str(p), flags=re.IGNORECASE) != None):
@@ -1122,7 +1064,8 @@ def getDataTypeInfo(source_graph, datatype):
         return False
     else:
         return {'label': label, 'hasUnit': hasUnit, 'datumType': typeURI, 'measureOf': measureOf, 'isAbout': isAbout,
-                'dataElement': str(URITail(s)), 'dataElementURI': str(s), 'description': description, 'prefix': prefix }
+                'dataElement': str(URITail(s)), 'dataElementURI': s, 'description': description, 'prefix': prefix,
+                'source_variable': source_variable}
 
 def getStatsCollectionForNode (rdf_graph, derivatives_node):
 
@@ -1225,13 +1168,12 @@ def getSoftwareAgents(rdf_graph):
     return agents
 
 def download_cde_files():
-    cde_dir = tempfile.gettempdir()
+        cde_dir = tempfile.gettempdir()
 
-    for url in Constants.CDE_FILE_LOCATIONS:
-        urlretrieve( url, "{}/{}".format(cde_dir, url.split('/')[-1] ) )
-
-    return cde_dir
+        for url in Constants.CDE_FILE_LOCATIONS:
+            urlretrieve(url, "{}/{}".format(cde_dir, url.split('/')[-1]))
 
+        return cde_dir
 
 def getCDEs(file_list=None):
 
@@ -1263,6 +1205,7 @@ def getCDEs(file_list=None):
         if (not cde_dir):
             cde_dir = download_cde_files()
 
+        # TODO: the list of file names should be it's own constant or derived from CDE_FILE_LOCATIONS
         file_list = [ ]
         for f in ['ants_cde.ttl', 'fs_cde.ttl', 'fsl_cde.ttl']:
             fname = '{}/{}'.format(cde_dir, f)
