@@ -132,20 +132,27 @@ def main(argv):
 
         #read in NIDM file
         project = read_nidm(args.nidm_file)
+        #with open("/Users/dbkeator/Downloads/test.ttl","w") as f:
+        #    f.write(project.serializeTurtle())
+
+
         #get list of session objects
         session_objs=project.get_sessions()
 
         #look at column_to_terms dictionary for NIDM URL for subject id  (Constants.NIDM_SUBJECTID)
         id_field=None
         for key, value in column_to_terms.items():
-            if Constants.NIDM_SUBJECTID._str == column_to_terms[key]['label']:
-                key_tuple = eval(key)
-                #id_field=key
-                id_field = key_tuple.variable
-                #make sure id_field is a string for zero-padded subject ids
-                #re-read data file with constraint that key field is read as string
-                df = pd.read_csv(args.csv_file,dtype={id_field : str})
-                break
+            if 'isAbout' in column_to_terms[key]:
+                for isabout_key,isabout_value in  column_to_terms[key]['isAbout'].items():
+                    if (isabout_key == 'url') or (isabout_key == '@id'):
+                        if (isabout_value == Constants.NIDM_SUBJECTID._uri):
+                            key_tuple = eval(key)
+                            #id_field=key
+                            id_field = key_tuple.variable
+                            #make sure id_field is a string for zero-padded subject ids
+                            #re-read data file with constraint that key field is read as string
+                            df = pd.read_csv(args.csv_file,dtype={id_field : str})
+                            break
 
         #if we couldn't find a subject ID field in column_to_terms, ask user
         if id_field is None:
@@ -186,7 +193,7 @@ def main(argv):
 
 
         for index,row in qres.iterrows():
-            logging.info("found existing participant %s \t %s" %(row[0],row[1]))
+            logging.info("participant in NIDM file %s \t %s" %(row[0],row[1]))
             #find row in CSV file with subject id matching agent from NIDM file
 
             #csv_row = df.loc[df[id_field]==type(df[id_field][0])(row[1])]
@@ -200,22 +207,28 @@ def main(argv):
             #then add this CSV assessment data to NIDM file, else skip it....
             if (not (len(csv_row.index)==0)):
 
+                logging.info("found participant in CSV file" )
+
+                # create a new session for this assessment
+                new_session=Session(project=project)
+
                 #NIDM document sesssion uuid
-                session_uuid = row[0]
+                #session_uuid = row[0]
 
                 #temporary list of string-based URIs of session objects from API
-                temp = [o.identifier._uri for o in session_objs]
+                #temp = [o.identifier._uri for o in session_objs]
                 #get session object from existing NIDM file that is associated with a specific subject id
                 #nidm_session = (i for i,x in enumerate([o.identifier._uri for o in session_objs]) if x == str(session_uuid))
-                nidm_session = session_objs[temp.index(str(session_uuid))]
+                #nidm_session = session_objs[temp.index(str(session_uuid))]
                 #for nidm_session in session_objs:
                 #    if nidm_session.identifier._uri == str(session_uuid):
                 #add an assessment acquisition for the phenotype data to session and associate with agent
-                acq=AssessmentAcquisition(session=nidm_session)
+                #acq=AssessmentAcquisition(session=nidm_session)
+                acq=AssessmentAcquisition(session=new_session)
                 #add acquisition entity for assessment
                 acq_entity = AssessmentObject(acquisition=acq)
                 #add qualified association with existing agent
-                acq.add_qualified_association(person=row[2],role=Constants.NIDM_PARTICIPANT)
+                acq.add_qualified_association(person=row[0],role=Constants.NIDM_PARTICIPANT)
 
                 # add git-annex info if exists
                 num_sources = addGitAnnexSources(obj=acq_entity,filepath=args.csv_file,bids_root=dirname(args.csv_file))
