@@ -5,20 +5,17 @@ import pytest
 import rdflib
 
 from nidm.experiment import Project, Session, AssessmentAcquisition, AssessmentObject, Acquisition, AcquisitionObject, Query
+from nidm.experiment.CDE import getCDEs
 from nidm.core import Constants
 from nidm.experiment.tools.rest import RestParser
 import os
 from pathlib import Path
 from rdflib import Graph, util, URIRef
-import json
-
-
-from prov.model import ProvAgent
-
 
 REST_TEST_FILE = './agent.ttl'
 BRAIN_VOL_FILES = ['./cmu_a.nidm.ttl', './caltech.nidm.ttl']
 OPENNEURO_FILES = ['ds000168.nidm.ttl']
+ALL_FILES = ['./cmu_a.nidm.ttl', './caltech.nidm.ttl', 'ds000168.nidm.ttl']
 OPENNEURO_PROJECT_URI = None
 OPENNEURO_SUB_URI = None
 
@@ -148,6 +145,36 @@ def makeTestFile(filename, params):
     with open("./agent.ttl", "w") as f:
         f.write(x)
 
+def test_uri_subject_list():
+    restParser = RestParser(output_format=RestParser.OBJECT_FORMAT)
+    result = restParser.run(ALL_FILES, '/subjects')
+
+    assert type(result) == dict
+    assert type(result['uuid']) == list
+    assert len(result['uuid']) > 10
+
+def test_uri_subject_list_with_fields():
+    restParser = RestParser(output_format=RestParser.OBJECT_FORMAT)
+    result = restParser.run(ALL_FILES, '/subjects?fields=ilx_0100400,MagneticFieldStrength') # ilx_0100400 "is about" age
+    assert type(result) == dict
+
+    assert type(result['uuid']) == list
+    assert len(result['uuid']) > 10
+
+    assert type(result['fields']) == dict
+    all_fields = []
+    for uuid in result['fields']:
+        assert type(result['fields']) == dict
+        for sub in result['fields']:
+            assert type(result['fields'][sub]) == dict
+            for activity in result['fields'][sub]:
+                all_fields.append(result['fields'][sub][activity].label)
+                if result['fields'][sub][activity].value != 'n/a':
+                    assert float(result['fields'][sub][activity].value) > 0
+                    assert float(result['fields'][sub][activity].value) < 125
+    assert 'age' in all_fields
+    assert 'MagneticFieldStrength' in all_fields
+
 def test_uri_project_list():
 
     kwargs={Constants.NIDM_PROJECT_NAME:"FBIRN_PhaseII",Constants.NIDM_PROJECT_IDENTIFIER:9610,Constants.NIDM_PROJECT_DESCRIPTION:"Test investigation"}
@@ -325,7 +352,6 @@ def test_GetParticipantDetails():
     # assert (runtime <  4)
 
 
-
 def test_CheckSubjectMatchesFilter():
     restParser = RestParser()
     if cmu_test_project_uuid:
@@ -364,10 +390,6 @@ def test_CheckSubjectMatchesFilter():
     assert Query.CheckSubjectMatchesFilter( BRAIN_VOL_FILES, project, subject, "instruments.AGE_AT_SCAN lt {}".format( older ) )
     assert (Query.CheckSubjectMatchesFilter( BRAIN_VOL_FILES, project, subject, "instruments.AGE_AT_SCAN gt {}".format( older) ) == False)
 
-    eq__format = "instruments.{} eq '{}'".format('WISC_IV_VOCAB_SCALED', 'nan')
-    assert Query.CheckSubjectMatchesFilter(BRAIN_VOL_FILES, project, subject, eq__format)
-    eq__format = "instruments.{} eq '{}'".format('WISC_IV_VOCAB_SCALED', 'not a match')
-    assert (Query.CheckSubjectMatchesFilter( BRAIN_VOL_FILES, project, subject, eq__format ) == False)
 
 def test_ExtremeFilters():
     restParser = RestParser(output_format=RestParser.OBJECT_FORMAT)
@@ -385,7 +407,6 @@ def test_ExtremeFilters():
     assert len(details['subjects']['uuid']) > 0
     assert len(details['data_elements']['uuid']) > 0
 
-
 def test_Filter_Flexibility():
     restParser = RestParser(output_format=RestParser.OBJECT_FORMAT)
     if cmu_test_project_uuid:
@@ -428,7 +449,7 @@ def test_CDEs():
         dir_parts.append("cde_dir")
         dir = "/".join(dir_parts)
 
-        graph = Query.getCDEs([
+        graph = getCDEs([
             "{}/ants_cde.ttl".format(dir),
             "{}/fs_cde.ttl".format(dir)
         ])
@@ -442,7 +463,7 @@ def test_CDEs():
         assert count == 1
 
     testrun()
-    Query.getCDEs.cache = None # clear the memory cache and try again
+    getCDEs.cache = None # clear the memory cache and try again
     testrun() # run a second time to test disk caching.
 
 def assess_one_col_output(txt_output):
@@ -560,6 +581,7 @@ def test_project_fields_deriv():
     fields_used = set( [ i.label for i in fv ]  )
     assert 'Brain Segmentation Volume (mm^3)' in fields_used
 
+
 def test_project_fields_instruments():
     rest_parser = RestParser(verbosity_level=0)
 
@@ -569,8 +591,9 @@ def test_project_fields_instruments():
     rest_parser.setOutputFormat(RestParser.OBJECT_FORMAT)
 
 
-    field = 'ncidb:Age'
-    project = rest_parser.run( BRAIN_VOL_FILES, "/projects/{}?fields={}".format(proj_uuid, field) )
+    field = 'age at scan'
+    uri = "/projects/{}?fields={}".format(proj_uuid,field)
+    project = rest_parser.run( BRAIN_VOL_FILES, uri)
 
     # edited by DBK to account for only field values being returned
     #assert( 'field_values' in project )
@@ -597,4 +620,22 @@ def test_project_fields_not_found():
     assert "error" in keys
 
 
+def test_GetProjectsComputedMetadata():
+
+    files = []
+
+    rest = RestParser()
+    rest.nidm_files = tuple(BRAIN_VOL_FILES)
+    meta_data = Query.GetProjectsMetadata(BRAIN_VOL_FILES)
+    rest.ExpandProjectMetaData(meta_data)
+    parsed = Query.compressForJSONResponse(meta_data)
+
+    for project_id in parsed['projects']:
+        if parsed['projects'][project_id][str(Constants.NIDM_PROJECT_NAME)] == "ABIDE CMU_a Site":
+            p3 = project_id
+    assert parsed['projects'][p3][str(Constants.NIDM_PROJECT_NAME)] == "ABIDE CMU_a Site"
+    assert parsed['projects'][p3][Query.matchPrefix(str(Constants.NIDM_NUMBER_OF_SUBJECTS))] == 14
+    assert parsed['projects'][p3]["age_min"] == 21.0
+    assert parsed['projects'][p3]["age_max"] == 33.0
+    assert set(parsed['projects'][p3][str(Constants.NIDM_GENDER)]) == set(['1', '2'])
 
